unit HexDumpUnit;

// ---------------------------------------------------
// Utility-Unit zur Ausgabe von Hex-Dumps
// Version 1.2 (C)2019-2021 arminlinder@arminlinder.de
// ---------------------------------------------------

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls;

function HexDump(const Data; size: integer):String;     // Dump tp plain string
procedure HexDump(const Data; size: integer; s: TStrings);  // Formatted 16 Byte chunks
procedure HexDumpToMemo(const Data; Size: integer; const Memo: TMemo; Caption: string = '');  // Formatted 16 Byte chunks to memo

implementation

const
  SepHex = ' ';  // Ascii Hex byte spacing character
  SepAsc = ' ';  // ASCII begin of dump block
  NonAsc = '.';  // non-printable character placeholder
  PrintableChars = [32..127]; // printable ASCII chars

function HexDump(const Data; size: integer): String;

var
  i:integer;
  buff: array[0..$FFFF] of byte Absolute Data;

begin
  Result := '';
  for i := 0 to Size - 1 do
    result := result + IntToHex(buff[i], 2) + SepHex;
  if Size > 0 then result := copy(Result,1,length(result)-1);
end;

procedure HexDump(const Data; Size: integer; s: TStrings);

// convert array of byte into hex string + ASCII string dump
// see: https://stackoverflow.com/questions/6304896/hex-view-of-a-file

var
  i: integer;
  hexDat, ascDat: string;
  buff: array[0..$FFFF] of byte Absolute Data;

begin
  hexDat := '';
  ascDat := '';
  for i := 0 to Size - 1 do
  begin
    hexDat := hexDat + IntToHex(buff[i], 2) + SepHex;
    if ((buff[i] > 31) and (buff[i] in PrintableChars)) then
      ascDat := ascDat + char(buff[i])
    else
      ascDat := ascDat + NonAsc;
    if (((i + 1) mod 16) <> 0) and (((i + 1) mod 8) = 0) then
      hexDat := hexDat + SepHex;
    if ((i + 1) mod 16) = 0 then
    begin
      s.Add(hexdat + SepAsc + ascdat);
      hexdat := '';
      ascdat := '';
    end;
  end;
  if (Size mod 16) <> 0 then
  begin
    if (Size mod 16) < 8 then
      hexDat := hexDat + StringOfChar(' ', (8 - (Size mod 8)) * 2) + SepHex + StringOfChar(' ', 16)
    else
      hexDat := hexDat + StringOfChar(' ', (16 - (Size mod 16)) * 2);
    s.Add(hexDat + SepAsc + ascDat);
  end;
end;

procedure HexDumpToMemo(const Data; Size: integer; const Memo: TMemo; Caption: string = '');

begin
  If Caption <> '' then
    begin
    Memo.Lines.Add(StringOfChar('-', 66));
    Memo.Lines.Add(Caption);
    end;
  Memo.Lines.Add(StringOfChar('-',66));
  HexDump(Data, Size, Memo.Lines);
  Memo.Lines.Add(Format('%d Bytes at address $%p',[Size,@Data]));
end;

end.

